@@ -136,8 +136,22 @@ class Volume(base.ResourceBase):
         if (payload and payload.get(_OAT_PROP)
                 == res_cons.ApplyTime.IMMEDIATE.value):
             blocking = True
-        r = self._conn.delete(self._path, data=payload, blocking=blocking,
-                              timeout=timeout)
+        try:
+            r = self._conn.delete(self._path, data=payload,
+                                  blocking=blocking, timeout=timeout)
+        except exceptions.ServerSideError as exc:
+            if (_OAT_PROP in str(exc.message)
+                    and 'SYS029' in str(exc.message)):
+                LOG.debug('Retry volume delete without %(prop)s for %(path)s '
+                          'because got error: %(err)s',
+                          {'prop': _OAT_PROP,
+                           'path': self._path,
+                           'err': exc})
+                payload.pop(_OAT_PROP)
+                r = self._conn.delete(self._path, data=payload,
+                                      blocking=blocking, timeout=timeout)
+            else:
+                raise exc
         return r
 
     def delete(self, payload=None, apply_time=None, timeout=500):
