@@ -23,9 +23,9 @@ from sushy import exceptions
 from sushy.resources.chassis import chassis
 from sushy.resources import constants as res_cons
 from sushy.resources.manager import manager
+from sushy.resources.manager import virtual_media
 from sushy.resources.oem import fake
 from sushy.resources.system import bios
-from sushy.resources.system import mappings as sys_map
 from sushy.resources.system import processor
 from sushy.resources.system import secure_boot
 from sushy.resources.system import simple_storage
@@ -36,7 +36,7 @@ from sushy.tests.unit import base
 class SystemTestCase(base.TestCase):
 
     def setUp(self):
-        super(SystemTestCase, self).setUp()
+        super().setUp()
         self.conn = mock.Mock()
         self.conn.get.return_value.headers = {'Allow': 'GET,HEAD'}
         with open('sushy/tests/unit/json_samples/system.json') as f:
@@ -47,6 +47,8 @@ class SystemTestCase(base.TestCase):
         self.sys_inst = system.System(
             self.conn, '/redfish/v1/Systems/437XR1138R2',
             redfish_version='1.0.2')
+        self.sys_inst._get_etag = mock.Mock()
+        self.sys_inst._get_etag.return_value = '81802dbf61beb0bd'
 
     def test__parse_attributes(self):
         self.sys_inst._parse_attributes(self.json_doc)
@@ -56,23 +58,21 @@ class SystemTestCase(base.TestCase):
         self.assertEqual('Web Front End node', self.sys_inst.description)
         self.assertEqual('web483', self.sys_inst.hostname)
         self.assertEqual('437XR1138R2', self.sys_inst.identity)
-        self.assertEqual(sushy.INDICATOR_LED_OFF,
+        self.assertEqual(sushy.IndicatorLED.OFF,
                          self.sys_inst.indicator_led)
         self.assertEqual('Contoso', self.sys_inst.manufacturer)
         self.assertEqual('WebFrontEnd483', self.sys_inst.name)
         self.assertEqual('224071-J23', self.sys_inst.part_number)
         self.assertEqual('437XR1138R2', self.sys_inst.serial_number)
         self.assertEqual('8675309', self.sys_inst.sku)
-        self.assertEqual(sushy.SYSTEM_TYPE_PHYSICAL,
-                         self.sys_inst.system_type)
+        self.assertEqual(sushy.SystemType.PHYSICAL, self.sys_inst.system_type)
         self.assertEqual('38947555-7742-3448-3784-823347823834',
                          self.sys_inst.uuid)
-        self.assertEqual(res_cons.STATE_ENABLED, self.sys_inst.status.state)
-        self.assertEqual(res_cons.HEALTH_OK, self.sys_inst.status.health)
-        self.assertEqual(res_cons.HEALTH_OK,
+        self.assertEqual(res_cons.State.ENABLED, self.sys_inst.status.state)
+        self.assertEqual(res_cons.Health.OK, self.sys_inst.status.health)
+        self.assertEqual(res_cons.Health.OK,
                          self.sys_inst.status.health_rollup)
-        self.assertEqual(sushy.SYSTEM_POWER_STATE_ON,
-                         self.sys_inst.power_state)
+        self.assertEqual(sushy.PowerState.ON, self.sys_inst.power_state)
         self.assertEqual(96, self.sys_inst.memory_summary.size_gib)
         self.assertEqual("OK", self.sys_inst.memory_summary.health)
         self.assertIsNotNone(self.sys_inst.maintenance_window)
@@ -83,42 +83,51 @@ class SystemTestCase(base.TestCase):
                          .maintenance_window_start_time)
         for oem_vendor in self.sys_inst.oem_vendors:
             self.assertIn(oem_vendor, ('Contoso', 'Chipwise'))
+        self.assertEqual(sushy.BootProgressStates.OS_RUNNING,
+                         self.sys_inst.boot_progress.last_state)
+        self.assertEqual(66,
+                         self.sys_inst.boot_progress.last_boot_seconds_count)
+        self.assertEqual(parser.parse('2017-05-03T23:12:37-05:00'),
+                         self.sys_inst.boot_progress.last_state_updated_at)
+        self.assertEqual("OS foo running.",
+                         self.sys_inst.boot_progress.oem_last_state)
 
     def test__parse_attributes_return(self):
         attributes = self.sys_inst._parse_attributes(self.json_doc)
 
         # Test that various types are returned correctly
         self.assertEqual('Chicago-45Z-2381', attributes.get('asset_tag'))
-        self.assertEqual(sushy.INDICATOR_LED_OFF,
+        self.assertEqual(sushy.IndicatorLED.OFF,
                          attributes.get('indicator_led'))
-        self.assertEqual({'health': res_cons.HEALTH_OK,
-                          'health_rollup': res_cons.HEALTH_OK,
-                          'state': res_cons.STATE_ENABLED},
+        self.assertEqual({'health': res_cons.Health.OK,
+                          'health_rollup': res_cons.Health.OK,
+                          'state': res_cons.State.ENABLED},
                          attributes.get('status'))
         self.assertEqual({'maintenance_window_duration_in_seconds': 1,
                           'maintenance_window_start_time':
                           parser.parse('2016-03-07T14:44:30-05:05')},
                          attributes.get('maintenance_window'))
-        self.assertEqual({'reset': {'allowed_values':
-                         ['On', 'ForceOff', 'GracefulShutdown',
-                             'GracefulRestart', 'ForceRestart', 'Nmi',
-                             'ForceOn', 'PushPowerButton'],
-                         'operation_apply_time_support':
-                         {'_maintenance_window_resource':
-                             {'resource_uri':
-                                 '/redfish/v1/Systems/437XR1138R2'},
-                          'maintenance_window_duration_in_seconds': 600,
-                          'maintenance_window_start_time':
-                              parser.parse('2017-05-03T23:12:37-05:00'),
-                          'supported_values':
-                              ['Immediate', 'AtMaintenanceWindowStart'],
-                          'mapped_supported_values':
-                              [res_cons.APPLY_TIME_IMMEDIATE,
-                               res_cons.APPLY_TIME_MAINT_START]},
-                         'target_uri':
-                         '/redfish/v1/Systems/437XR1138R2/Actions/'
-                         'ComputerSystem.Reset'}},
-                         attributes.get('_actions'))
+        self.assertEqual(
+            {'reset': {'allowed_values':
+                       ['On', 'ForceOff', 'GracefulShutdown',
+                        'GracefulRestart', 'ForceRestart', 'Nmi',
+                        'ForceOn', 'PushPowerButton'],
+                       'operation_apply_time_support':
+                       {'_maintenance_window_resource':
+                        {'resource_uri':
+                         '/redfish/v1/Systems/437XR1138R2'},
+                        'maintenance_window_duration_in_seconds': 600,
+                        'maintenance_window_start_time':
+                        parser.parse('2017-05-03T23:12:37-05:00'),
+                        'supported_values':
+                        ['Immediate', 'AtMaintenanceWindowStart'],
+                        'mapped_supported_values':
+                        [res_cons.ApplyTime.IMMEDIATE,
+                         res_cons.ApplyTime.AT_MAINTENANCE_WINDOW_START]},
+                       'target_uri':
+                       '/redfish/v1/Systems/437XR1138R2/Actions/'
+                       'ComputerSystem.Reset'}},
+            attributes.get('_actions'))
 
     def test__parse_attributes_missing_actions(self):
         self.sys_inst.json.pop('Actions')
@@ -177,14 +186,14 @@ class SystemTestCase(base.TestCase):
 
     def test_get_allowed_reset_system_values(self):
         values = self.sys_inst.get_allowed_reset_system_values()
-        expected = set([sushy.RESET_GRACEFUL_SHUTDOWN,
-                        sushy.RESET_GRACEFUL_RESTART,
-                        sushy.RESET_FORCE_RESTART,
-                        sushy.RESET_FORCE_OFF,
-                        sushy.RESET_FORCE_ON,
-                        sushy.RESET_ON,
-                        sushy.RESET_NMI,
-                        sushy.RESET_PUSH_POWER_BUTTON])
+        expected = set([sushy.ResetType.GRACEFUL_SHUTDOWN,
+                        sushy.ResetType.GRACEFUL_RESTART,
+                        sushy.ResetType.FORCE_RESTART,
+                        sushy.ResetType.FORCE_OFF,
+                        sushy.ResetType.FORCE_ON,
+                        sushy.ResetType.ON,
+                        sushy.ResetType.NMI,
+                        sushy.ResetType.PUSH_POWER_BUTTON])
         self.assertEqual(expected, values)
         self.assertIsInstance(values, set)
 
@@ -194,14 +203,18 @@ class SystemTestCase(base.TestCase):
         self.sys_inst._actions.reset.allowed_values = {}
         values = self.sys_inst.get_allowed_reset_system_values()
         # Assert it returns all values if it can't get the specific ones
-        expected = set([sushy.RESET_GRACEFUL_SHUTDOWN,
-                        sushy.RESET_GRACEFUL_RESTART,
-                        sushy.RESET_FORCE_RESTART,
-                        sushy.RESET_FORCE_OFF,
-                        sushy.RESET_FORCE_ON,
-                        sushy.RESET_ON,
-                        sushy.RESET_NMI,
-                        sushy.RESET_PUSH_POWER_BUTTON])
+        expected = set([sushy.ResetType.GRACEFUL_SHUTDOWN,
+                        sushy.ResetType.GRACEFUL_RESTART,
+                        sushy.ResetType.FORCE_RESTART,
+                        sushy.ResetType.FORCE_OFF,
+                        sushy.ResetType.FORCE_ON,
+                        sushy.ResetType.ON,
+                        sushy.ResetType.NMI,
+                        sushy.ResetType.PUSH_POWER_BUTTON,
+                        sushy.ResetType.POWER_CYCLE,
+                        sushy.ResetType.SUSPEND,
+                        sushy.ResetType.RESUME,
+                        sushy.ResetType.PAUSE])
         self.assertEqual(expected, values)
         self.assertIsInstance(values, set)
         self.assertEqual(1, mock_log.call_count)
@@ -211,8 +224,8 @@ class SystemTestCase(base.TestCase):
         self.assertIsNotNone(support)
         self.assertEqual(['Immediate', 'AtMaintenanceWindowStart'],
                          support.supported_values)
-        self.assertEqual([res_cons.APPLY_TIME_IMMEDIATE,
-                          res_cons.APPLY_TIME_MAINT_START],
+        self.assertEqual([res_cons.ApplyTime.IMMEDIATE,
+                          res_cons.ApplyTime.AT_MAINTENANCE_WINDOW_START],
                          support.mapped_supported_values)
         self.assertEqual(parser.parse('2017-05-03T23:12:37-05:00'),
                          support.maintenance_window_start_time)
@@ -221,7 +234,7 @@ class SystemTestCase(base.TestCase):
                          support._maintenance_window_resource.resource_uri)
 
     def test_reset_system(self):
-        self.sys_inst.reset_system(sushy.RESET_FORCE_OFF)
+        self.sys_inst.reset_system(sushy.ResetType.FORCE_OFF)
         self.sys_inst._conn.post.assert_called_once_with(
             '/redfish/v1/Systems/437XR1138R2/Actions/ComputerSystem.Reset',
             data={'ResetType': 'ForceOff'})
@@ -232,16 +245,17 @@ class SystemTestCase(base.TestCase):
 
     def test_get_allowed_system_boot_source_values(self):
         values = self.sys_inst.get_allowed_system_boot_source_values()
-        expected = set([sushy.BOOT_SOURCE_TARGET_NONE,
-                        sushy.BOOT_SOURCE_TARGET_PXE,
-                        sushy.BOOT_SOURCE_TARGET_CD,
-                        sushy.BOOT_SOURCE_TARGET_USB,
-                        sushy.BOOT_SOURCE_TARGET_HDD,
-                        sushy.BOOT_SOURCE_TARGET_BIOS_SETUP,
-                        sushy.BOOT_SOURCE_TARGET_UTILITIES,
-                        sushy.BOOT_SOURCE_TARGET_DIAGS,
-                        sushy.BOOT_SOURCE_TARGET_SD_CARD,
-                        sushy.BOOT_SOURCE_TARGET_UEFI_TARGET])
+        expected = set([sushy.BootSource.NONE,
+                        sushy.BootSource.PXE,
+                        sushy.BootSource.CD,
+                        sushy.BootSource.USB,
+                        sushy.BootSource.HDD,
+                        sushy.BootSource.BIOS_SETUP,
+                        sushy.BootSource.UTILITIES,
+                        sushy.BootSource.DIAGS,
+                        sushy.BootSource.SD_CARD,
+                        sushy.BootSource.UEFI_TARGET,
+                        sushy.BootSource.UEFI_HTTP])
         self.assertEqual(expected, values)
         self.assertIsInstance(values, set)
 
@@ -251,65 +265,67 @@ class SystemTestCase(base.TestCase):
         self.sys_inst.boot.allowed_values = None
         values = self.sys_inst.get_allowed_system_boot_source_values()
         # Assert it returns all values if it can't get the specific ones
-        expected = set([sushy.BOOT_SOURCE_TARGET_NONE,
-                        sushy.BOOT_SOURCE_TARGET_PXE,
-                        sushy.BOOT_SOURCE_TARGET_CD,
-                        sushy.BOOT_SOURCE_TARGET_USB,
-                        sushy.BOOT_SOURCE_TARGET_USB_CD,
-                        sushy.BOOT_SOURCE_TARGET_HDD,
-                        sushy.BOOT_SOURCE_TARGET_BIOS_SETUP,
-                        sushy.BOOT_SOURCE_TARGET_UTILITIES,
-                        sushy.BOOT_SOURCE_TARGET_DIAGS,
-                        sushy.BOOT_SOURCE_TARGET_SD_CARD,
-                        sushy.BOOT_SOURCE_TARGET_FLOPPY,
-                        sushy.BOOT_SOURCE_TARGET_UEFI_TARGET,
-                        sushy.BOOT_SOURCE_TARGET_UEFI_SHELL,
-                        sushy.BOOT_SOURCE_TARGET_UEFI_HTTP])
+        expected = set([sushy.BootSource.NONE,
+                        sushy.BootSource.PXE,
+                        sushy.BootSource.CD,
+                        sushy.BootSource.USB,
+                        sushy.BootSource.HDD,
+                        sushy.BootSource.BIOS_SETUP,
+                        sushy.BootSource.UTILITIES,
+                        sushy.BootSource.DIAGS,
+                        sushy.BootSource.SD_CARD,
+                        sushy.BootSource.FLOPPY,
+                        sushy.BootSource.UEFI_TARGET,
+                        sushy.BootSource.UEFI_SHELL,
+                        sushy.BootSource.UEFI_HTTP,
+                        sushy.BootSource.REMOTE_DRIVE,
+                        sushy.BootSource.UEFI_BOOT_NEXT,
+                        sushy.BootSource.USB_CD])
         self.assertEqual(expected, values)
         self.assertIsInstance(values, set)
         self.assertEqual(1, mock_log.call_count)
 
     def test_set_system_boot_options(self):
         self.sys_inst.set_system_boot_options(
-            sushy.BOOT_SOURCE_TARGET_PXE,
-            enabled=sushy.BOOT_SOURCE_ENABLED_CONTINUOUS,
-            mode=sushy.BOOT_SOURCE_MODE_UEFI)
+            sushy.BootSource.PXE,
+            enabled=sushy.BootSourceOverrideEnabled.CONTINUOUS,
+            mode=sushy.BootSourceOverrideMode.UEFI)
         self.sys_inst._conn.patch.assert_called_once_with(
             '/redfish/v1/Systems/437XR1138R2',
             data={'Boot': {'BootSourceOverrideEnabled': 'Continuous',
                            'BootSourceOverrideTarget': 'Pxe',
                            'BootSourceOverrideMode': 'UEFI'}},
-            etag=None)
+            etag='81802dbf61beb0bd')
 
     def test_set_system_boot_options_no_mode_specified(self):
         self.sys_inst.set_system_boot_options(
-            sushy.BOOT_SOURCE_TARGET_HDD,
-            enabled=sushy.BOOT_SOURCE_ENABLED_ONCE)
+            sushy.BootSource.HDD,
+            enabled=sushy.BootSourceOverrideEnabled.ONCE)
         self.sys_inst._conn.patch.assert_called_once_with(
             '/redfish/v1/Systems/437XR1138R2',
             data={'Boot': {'BootSourceOverrideEnabled': 'Once',
                            'BootSourceOverrideTarget': 'Hdd'}},
-            etag=None)
+            etag='81802dbf61beb0bd')
 
     def test_set_system_boot_options_no_target_specified(self):
         self.sys_inst.set_system_boot_options(
-            enabled=sushy.BOOT_SOURCE_ENABLED_CONTINUOUS,
-            mode=sushy.BOOT_SOURCE_MODE_UEFI)
+            enabled=sushy.BootSourceOverrideEnabled.CONTINUOUS,
+            mode=sushy.BootSourceOverrideMode.UEFI)
         self.sys_inst._conn.patch.assert_called_once_with(
             '/redfish/v1/Systems/437XR1138R2',
             data={'Boot': {'BootSourceOverrideEnabled': 'Continuous',
                            'BootSourceOverrideMode': 'UEFI'}},
-            etag=None)
+            etag='81802dbf61beb0bd')
 
     def test_set_system_boot_options_no_freq_specified(self):
         self.sys_inst.set_system_boot_options(
-            target=sushy.BOOT_SOURCE_TARGET_PXE,
-            mode=sushy.BOOT_SOURCE_MODE_UEFI)
+            target=sushy.BootSource.PXE,
+            mode=sushy.BootSourceOverrideMode.UEFI)
         self.sys_inst._conn.patch.assert_called_once_with(
             '/redfish/v1/Systems/437XR1138R2',
             data={'Boot': {'BootSourceOverrideTarget': 'Pxe',
                            'BootSourceOverrideMode': 'UEFI'}},
-            etag=None)
+            etag='81802dbf61beb0bd')
 
     def test_set_system_boot_options_nothing_specified(self):
         self.sys_inst.set_system_boot_options()
@@ -323,11 +339,10 @@ class SystemTestCase(base.TestCase):
     def test_set_system_boot_options_invalid_enabled(self):
         with self.assertRaisesRegex(
             exceptions.InvalidParameterValueError,
-            '"enabled" value.*{0}'.format(
-                list(sys_map.BOOT_SOURCE_ENABLED_MAP_REV))):
+            f'"enabled" value.*{list(sushy.BootSourceOverrideEnabled)}'):
 
             self.sys_inst.set_system_boot_options(
-                sushy.BOOT_SOURCE_TARGET_HDD,
+                sushy.BootSource.HDD,
                 enabled='invalid-enabled')
 
     def test_set_system_boot_options_supermicro_usb_cd_boot(self):
@@ -337,27 +352,44 @@ class SystemTestCase(base.TestCase):
 
         self.sys_inst.manufacturer = "supermicro"
         self.sys_inst.set_system_boot_options(
-            target=sushy.BOOT_SOURCE_TARGET_CD,
-            enabled=sushy.BOOT_SOURCE_ENABLED_ONCE)
+            target=sushy.BootSource.CD,
+            enabled=sushy.BootSourceOverrideEnabled.ONCE)
 
         self.sys_inst._conn.patch.assert_called_once_with(
             '/redfish/v1/Systems/437XR1138R2',
             data={'Boot': {'BootSourceOverrideEnabled': 'Once',
                            'BootSourceOverrideTarget': 'UsbCd'}},
-            etag=None)
+            etag='81802dbf61beb0bd')
 
-    def test_set_system_boot_options_supermicro_no_usb_cd_boot(self):
+    def test_set_system_boot_options_supermicro_usb_cd_boot_ars111glnhr(self):
+        (self.json_doc["Boot"]
+         ["BootSourceOverrideTarget@Redfish.AllowableValues"]).append("UsbCd")
+        self.sys_inst._parse_attributes(self.json_doc)
 
         self.sys_inst.manufacturer = "supermicro"
+        self.sys_inst.model = "ARS-111GL-NHR"
         self.sys_inst.set_system_boot_options(
-            target=sushy.BOOT_SOURCE_TARGET_CD,
-            enabled=sushy.BOOT_SOURCE_ENABLED_ONCE)
+            target=sushy.BootSource.CD,
+            enabled=sushy.BootSourceOverrideEnabled.ONCE)
 
         self.sys_inst._conn.patch.assert_called_once_with(
             '/redfish/v1/Systems/437XR1138R2',
             data={'Boot': {'BootSourceOverrideEnabled': 'Once',
                            'BootSourceOverrideTarget': 'Cd'}},
-            etag=None)
+            etag='81802dbf61beb0bd')
+
+    def test_set_system_boot_options_supermicro_no_usb_cd_boot(self):
+
+        self.sys_inst.manufacturer = "supermicro"
+        self.sys_inst.set_system_boot_options(
+            target=sushy.BootSource.CD,
+            enabled=sushy.BootSourceOverrideEnabled.ONCE)
+
+        self.sys_inst._conn.patch.assert_called_once_with(
+            '/redfish/v1/Systems/437XR1138R2',
+            data={'Boot': {'BootSourceOverrideEnabled': 'Once',
+                           'BootSourceOverrideTarget': 'Cd'}},
+            etag='81802dbf61beb0bd')
 
     def test_set_system_boot_options_settings_resource_nokia(self):
         with open('sushy/tests/unit/json_samples/settings-nokia.json') as f:
@@ -376,8 +408,8 @@ class SystemTestCase(base.TestCase):
         self.conn.get.side_effect = [get_settings, get_system]
 
         self.sys_inst.set_system_boot_options(
-            target=sushy.BOOT_SOURCE_TARGET_CD,
-            enabled=sushy.BOOT_SOURCE_ENABLED_ONCE)
+            target=sushy.BootSource.CD,
+            enabled=sushy.BootSourceOverrideEnabled.ONCE)
 
         self.sys_inst._conn.patch.assert_called_once_with(
             '/redfish/v1/Systems/Self/SD',
@@ -404,8 +436,8 @@ class SystemTestCase(base.TestCase):
         self.conn.get.return_value.json.return_value = settings_body
 
         self.sys_inst.set_system_boot_options(
-            target=sushy.BOOT_SOURCE_TARGET_CD,
-            enabled=sushy.BOOT_SOURCE_ENABLED_ONCE)
+            target=sushy.BootSource.CD,
+            enabled=sushy.BootSourceOverrideEnabled.ONCE)
 
         self.sys_inst._conn.patch.assert_called_once_with(
             '/redfish/v1/Systems/437XR1138R2/BIOS/Settings',
@@ -435,8 +467,8 @@ class SystemTestCase(base.TestCase):
         self.conn.get.side_effect = [get_settings, get_system]
 
         self.sys_inst.set_system_boot_options(
-            target=sushy.BOOT_SOURCE_TARGET_CD,
-            enabled=sushy.BOOT_SOURCE_ENABLED_ONCE)
+            target=sushy.BootSource.CD,
+            enabled=sushy.BootSourceOverrideEnabled.ONCE)
 
         self.sys_inst._conn.patch.assert_called_once_with(
             '/redfish/v1/Systems/1',
@@ -466,27 +498,107 @@ class SystemTestCase(base.TestCase):
         self.conn.get.side_effect = [get_settings, get_system]
 
         self.sys_inst.set_system_boot_options(
-            mode=sushy.BOOT_SOURCE_MODE_UEFI)
+            mode=sushy.BootSourceOverrideMode.UEFI)
 
         self.sys_inst._conn.patch.assert_called_with(
             '/redfish/v1/Systems/Self/SD',
             data={'Boot': {'BootSourceOverrideMode': 'UEFI'}},
             etag='"3d7b838291941d"')
 
+    def test_set_system_boot_options_httpbooturi(self):
+        self.sys_inst.set_system_boot_options(
+            sushy.BootSource.UEFI_HTTP,
+            enabled=sushy.BootSourceOverrideEnabled.ONCE,
+            mode=sushy.BootSourceOverrideMode.UEFI,
+            http_boot_uri='http://test.lan/test_image.iso'
+            )
+        self.sys_inst._conn.patch.assert_called_once_with(
+            '/redfish/v1/Systems/437XR1138R2',
+            data={'Boot': {'BootSourceOverrideTarget': 'UefiHttp',
+                           'BootSourceOverrideEnabled': 'Once',
+                           'BootSourceOverrideMode': 'UEFI',
+                           'HttpBootUri': 'http://test.lan/test_image.iso'}},
+            etag=mock.ANY)
+
+    def test_set_system_boot_options_httpboot_settings(self):
+
+        with open('sushy/tests/unit/json_samples/settings-nokia.json') as f:
+            settings_obj = json.load(f)
+
+        self.json_doc.update(settings_obj)
+        self.sys_inst._parse_attributes(self.json_doc)
+
+        with open('sushy/tests/unit/json_samples/'
+                  'settings-body-nokia.json') as f:
+            settings_body = json.load(f)
+
+        get_settings = mock.MagicMock(headers={'ETag': '"3d7b838291941d"'})
+        get_settings.json.return_value = settings_body
+        get_system = mock.MagicMock(headers={'ETag': '"222"'})
+        self.conn.get.side_effect = [get_settings, get_system]
+
+        self.sys_inst.set_system_boot_options(
+            sushy.BootSource.UEFI_HTTP,
+            enabled=sushy.BootSourceOverrideEnabled.ONCE,
+            http_boot_uri='http://test.lan/test_image.iso'
+            )
+        self.sys_inst._conn.patch.assert_called_once_with(
+            '/redfish/v1/Systems/Self/SD',
+            data={'Boot': {'BootSourceOverrideTarget': 'UefiHttp',
+                           'BootSourceOverrideEnabled': 'Once',
+                           'HttpBootUri': 'http://test.lan/test_image.iso'}},
+            etag=mock.ANY)
+
+    def test_set_system_boot_options_httpboot(self):
+        self.sys_inst.set_system_boot_options(
+            sushy.BootSource.UEFI_HTTP,
+            enabled=sushy.BootSourceOverrideEnabled.ONCE,
+            mode=sushy.BootSourceOverrideMode.UEFI
+            )
+        self.sys_inst._conn.patch.assert_called_once_with(
+            '/redfish/v1/Systems/437XR1138R2',
+            data={'Boot': {'BootSourceOverrideTarget': 'UefiHttp',
+                           'BootSourceOverrideEnabled': 'Once',
+                           'BootSourceOverrideMode': 'UEFI',
+                           'HttpBootUri': None}},
+            etag=mock.ANY)
+
+    def test_set_system_boot_options_httpboot_unset(self):
+        self.sys_inst._settings = mock.Mock()
+        self.sys_inst._settings.resource_uri = 'meow'
+        settings_body = json.dumps(
+            {'Boot': {'HttpBootUri': 'http://foo.bar'}}
+        )
+
+        get_settings = mock.MagicMock()
+        get_settings.json.return_value = settings_body
+        self.conn.get.side_effect = get_settings
+
+        self.sys_inst.set_system_boot_options(
+            sushy.BootSource.HDD,
+            mode=sushy.BootSourceOverrideMode.UEFI
+            )
+        self.sys_inst._conn.patch.assert_called_once_with(
+            '/redfish/v1/Systems/437XR1138R2',
+            data={'Boot': {'BootSourceOverrideTarget': 'Hdd',
+                           'BootSourceOverrideMode': 'UEFI',
+                           'HttpBootUri': None}},
+            etag=mock.ANY)
+
     def test_set_system_boot_source(self):
         self.sys_inst.set_system_boot_source(
-            sushy.BOOT_SOURCE_TARGET_PXE,
-            enabled=sushy.BOOT_SOURCE_ENABLED_CONTINUOUS,
-            mode=sushy.BOOT_SOURCE_MODE_UEFI)
+            sushy.BootSource.PXE,
+            enabled=sushy.BootSourceOverrideEnabled.CONTINUOUS,
+            mode=sushy.BootSourceOverrideMode.UEFI)
         self.sys_inst._conn.patch.assert_called_once_with(
             '/redfish/v1/Systems/437XR1138R2',
             data={'Boot': {'BootSourceOverrideEnabled': 'Continuous',
                            'BootSourceOverrideTarget': 'Pxe',
                            'BootSourceOverrideMode': 'UEFI'}},
-            etag=None)
+            etag='81802dbf61beb0bd')
 
     def test_set_system_boot_source_with_etag(self):
-        self.conn.get.return_value.headers = {'ETag': '"3d7b838291941d"'}
+        self.conn.get.return_value.headers = {'ETag': '"81802dbf61beb0bd"'}
         self.sys_inst.set_system_boot_source(
             sushy.BOOT_SOURCE_TARGET_PXE,
             enabled=sushy.BOOT_SOURCE_ENABLED_CONTINUOUS,
@@ -496,17 +608,27 @@ class SystemTestCase(base.TestCase):
             data={'Boot': {'BootSourceOverrideEnabled': 'Continuous',
                            'BootSourceOverrideTarget': 'Pxe',
                            'BootSourceOverrideMode': 'UEFI'}},
-            etag='"3d7b838291941d"')
+            etag="81802dbf61beb0bd")
 
     def test_set_system_boot_source_no_mode_specified(self):
         self.sys_inst.set_system_boot_source(
-            sushy.BOOT_SOURCE_TARGET_HDD,
-            enabled=sushy.BOOT_SOURCE_ENABLED_ONCE)
+            sushy.BootSource.HDD,
+            enabled=sushy.BootSourceOverrideEnabled.ONCE)
         self.sys_inst._conn.patch.assert_called_once_with(
             '/redfish/v1/Systems/437XR1138R2',
             data={'Boot': {'BootSourceOverrideEnabled': 'Once',
                            'BootSourceOverrideTarget': 'Hdd'}},
-            etag=None)
+            etag='81802dbf61beb0bd')
+
+    def test_set_system_boot_unsets_http_boot_uri(self):
+        self.sys_inst.set_system_boot_source(
+            sushy.BootSource.HDD,
+            enabled=sushy.BootSourceOverrideEnabled.ONCE)
+        self.sys_inst._conn.patch.assert_called_once_with(
+            '/redfish/v1/Systems/437XR1138R2',
+            data={'Boot': {'BootSourceOverrideEnabled': 'Once',
+                           'BootSourceOverrideTarget': 'Hdd'}},
+            etag=mock.ANY)
 
     def test_set_system_boot_source_invalid_target(self):
         self.assertRaises(exceptions.InvalidParameterValueError,
@@ -516,20 +638,20 @@ class SystemTestCase(base.TestCase):
     def test_set_system_boot_source_invalid_enabled(self):
         with self.assertRaisesRegex(
             exceptions.InvalidParameterValueError,
-            '"enabled" value.*{0}'.format(
-                list(sys_map.BOOT_SOURCE_ENABLED_MAP_REV))):
+            f'"enabled" value.*{list(sushy.BootSourceOverrideEnabled)}'):
 
             self.sys_inst.set_system_boot_source(
-                sushy.BOOT_SOURCE_TARGET_HDD,
+                sushy.BootSource.HDD,
                 enabled='invalid-enabled')
 
     def test_set_indicator_led(self):
         with mock.patch.object(
                 self.sys_inst, 'invalidate', autospec=True) as invalidate_mock:
-            self.sys_inst.set_indicator_led(sushy.INDICATOR_LED_BLINKING)
+            self.sys_inst.set_indicator_led(sushy.IndicatorLED.BLINKING)
             self.sys_inst._conn.patch.assert_called_once_with(
                 '/redfish/v1/Systems/437XR1138R2',
-                data={'IndicatorLED': 'Blinking'})
+                data={'IndicatorLED': 'Blinking'},
+                etag='81802dbf61beb0bd')
 
             invalidate_mock.assert_called_once_with()
 
@@ -645,10 +767,10 @@ class SystemTestCase(base.TestCase):
         # | WHEN |
         actual_processor_summary = self.sys_inst.processors.summary
         # | THEN |
-        self.assertEqual((16, sushy.PROCESSOR_ARCH_x86),
+        self.assertEqual((16, sushy.ProcessorArchitecture.X86),
                          actual_processor_summary)
         self.assertEqual(16, actual_processor_summary.count)
-        self.assertEqual(sushy.PROCESSOR_ARCH_x86,
+        self.assertEqual(sushy.ProcessorArchitecture.X86,
                          actual_processor_summary.architecture)
 
         # reset mock
@@ -676,7 +798,7 @@ class SystemTestCase(base.TestCase):
 
         actual_macs = self.sys_inst.ethernet_interfaces.summary
         expected_macs = (
-            {'12:44:6A:3B:04:11': res_cons.STATE_ENABLED})
+            {'12:44:6A:3B:04:11': res_cons.State.ENABLED})
         self.assertEqual(expected_macs, actual_macs)
 
     def test_bios(self):
@@ -812,6 +934,33 @@ class SystemTestCase(base.TestCase):
         self.assertEqual(
             '/redfish/v1/Managers/BMC', actual_managers[0].path)
 
+    def test_managers_retry(self):
+        with open('sushy/tests/unit/json_samples/'
+                  'system_managedby_missing_manager_present.json') as f:
+            settings_obj = json.load(f)
+        self.json_doc.update(settings_obj)
+        self.sys_inst._parse_attributes(self.json_doc)
+
+        # | GIVEN |
+        with open('sushy/tests/unit/json_samples/'
+                  'manager.json') as f:
+            self.conn.get.return_value.json.return_value = json.load(f)
+
+        # | WHEN & THEN |
+        actual_managers = self.sys_inst.managers
+        self.assertIsInstance(actual_managers[0], manager.Manager)
+        self.assertEqual(
+            '/redfish/v1/Managers/BMC', actual_managers[0].path)
+
+    def test_managers_missing_ref(self):
+        # | GIVEN |
+        del self.json_doc['Links']['ManagedBy']
+
+        # | WHEN & THEN |
+        with self.assertRaisesRegex(
+            exceptions.MissingAttributeError, 'attribute Links/ManagedBy'):
+            self.sys_inst.managers
+
     def test_chassis(self):
         # | GIVEN |
         with open('sushy/tests/unit/json_samples/'
@@ -833,11 +982,90 @@ class SystemTestCase(base.TestCase):
         self.assertIs(self.sys_inst, contoso_system_extn_inst._parent_resource)
         self.assertEqual('Contoso', contoso_system_extn_inst._vendor_id)
 
+    def test_no_virtual_media_attr(self):
+        with self.assertRaisesRegex(
+            exceptions.MissingAttributeError, 'attribute VirtualMedia'):
+            self.sys_inst.virtual_media
+
+
+class SystemWithVirtualMedia(base.TestCase):
+
+    def setUp(self):
+        super().setUp()
+        self.conn = mock.Mock()
+        with open('sushy/tests/unit/json_samples/'
+                  'systemv1_20.json') as f:
+            self.json_doc = json.load(f)
+
+        self.conn.get.return_value.json.return_value = self.json_doc
+
+        self.sys_inst = system.System(
+            self.conn, '/redfish/v1/Systems/437XR1138R2',
+            redfish_version='1.0.2')
+
+    def test_virtual_media(self):
+        # | GIVEN |
+        with open('sushy/tests/unit/json_samples/'
+                  'virtual_mediav1_6.json') as f:
+            virtual_media_json = json.load(f)
+
+        with open('sushy/tests/unit/json_samples/'
+                  'virtual_media_collectionv1_6.json') as f:
+            virtual_media_collection_json = json.load(f)
+
+        self.conn.get.return_value.json.side_effect = [
+            virtual_media_collection_json, virtual_media_json]
+
+        # | WHEN |
+        actual_virtual_media = self.sys_inst.virtual_media
+
+        # | THEN |
+        self.assertIsInstance(actual_virtual_media,
+                              virtual_media.VirtualMediaCollection)
+        self.assertEqual(actual_virtual_media.name, 'Virtual Media Services')
+
+        member = actual_virtual_media.get_member(
+            '/redfish/v1/Systems/437XR1138R2/VirtualMedia/CD1')
+
+        self.assertEqual(member.image_name, 'mymedia-read-only')
+        self.assertTrue(member.inserted)
+        self.assertFalse(member.write_protected)
+        self.assertTrue(member.verify_certificate)
+
+    def test_virtual_media_on_refresh(self):
+        # | GIVEN |
+        with open('sushy/tests/unit/json_samples/'
+                  'virtual_media_collectionv1_6.json') as f:
+            self.conn.get.return_value.json.return_value = json.load(f)
+
+        # | WHEN & THEN |
+        actual_virtual_media = self.sys_inst.virtual_media
+        self.assertIsInstance(actual_virtual_media,
+                              virtual_media.VirtualMediaCollection)
+
+        with open('sushy/tests/unit/json_samples/systemv1_20.json') as f:
+            self.conn.get.return_value.json.return_value = json.loads(f.read())
+
+        self.sys_inst.invalidate()
+        self.sys_inst.refresh(force=False)
+
+        self.assertTrue(actual_virtual_media._is_stale)
+
+        # | GIVEN |
+        with open('sushy/tests/unit/json_samples/'
+                  'virtual_media_collectionv1_6.json') as f:
+            self.conn.get.return_value.json.return_value = json.load(f)
+
+        # | WHEN & THEN |
+        self.assertIsInstance(self.sys_inst.virtual_media,
+                              virtual_media.VirtualMediaCollection)
+        self.assertFalse(actual_virtual_media._is_stale)
+
 
 class SystemCollectionTestCase(base.TestCase):
 
     def setUp(self):
-        super(SystemCollectionTestCase, self).setUp()
+        super().setUp()
         self.conn = mock.Mock()
         with open('sushy/tests/unit/json_samples/'
                   'system_collection.json') as f:
