@@ -15,6 +15,7 @@
 
 from http import client as http_client
 import json
+import time
 from unittest import mock
 
 import requests
@@ -398,7 +399,8 @@ class ConnectorOpTestCase(base.TestCase):
         self.auth.refresh_session.assert_called_with()
         self.auth.can_refresh_session.assert_called_with()
 
-    def test_connection_error(self):
+    @mock.patch.object(time, 'sleep', autospec=True)
+    def test_connection_error(self, mock_sleep):
         self.request.side_effect = requests.exceptions.ConnectionError
         self.assertRaises(exceptions.ConnectionError, self.conn._op, 'GET')
 
@@ -724,7 +726,8 @@ class ConnectorOpTestCase(base.TestCase):
             {'err': error_msg}
         )
 
-    def test__op_raises_connection_error(self):
+    @mock.patch.object(time, 'sleep', autospec=True)
+    def test__op_raises_connection_error(self, mock_sleep):
 
         exception_list = [
             requests.exceptions.ChunkedEncodingError,
@@ -894,6 +897,51 @@ class ConnectorOpTestCase(base.TestCase):
             blocking=False,
             timeout=60)
 
+    @mock.patch.object(connector.Connector, '_op', autospec=True)
+    def test_etag_handler_emptystring_etag(self, mock__op):
+        self.headers_no_etag = self.headers.copy()
+        target_uri = '/redfish/v1/Systems/1'
+        response_200 = mock.MagicMock(spec=requests.Response)
+        response_200.status_code = http_client.OK
+
+        data = {'Boot': {'BootSourceOverrideTarget': 'Cd',
+                         'BootSourceOverrideEnabled': 'Once'}}
+        self.conn._etag_handler(path=target_uri, headers=self.headers,
+                                data=data, etag='',
+                                blocking=False, timeout=60)
+        mock__op.assert_called_with(
+            self.conn,
+            "PATCH",
+            '/redfish/v1/Systems/1',
+            data={'Boot': {'BootSourceOverrideTarget': 'Cd',
+                           'BootSourceOverrideEnabled': 'Once'}},
+            headers=self.headers_no_etag,
+            blocking=False,
+            timeout=60)
+
+    @mock.patch.object(connector.Connector, '_op', autospec=True)
+    def test_etag_handler_emptystring_etag_from_headers(self, mock__op):
+        self.headers_no_etag = self.headers.copy()
+        self.headers['If-Match'] = ''
+        target_uri = '/redfish/v1/Systems/1'
+        response_200 = mock.MagicMock(spec=requests.Response)
+        response_200.status_code = http_client.OK
+
+        data = {'Boot': {'BootSourceOverrideTarget': 'Cd',
+                         'BootSourceOverrideEnabled': 'Once'}}
+        self.conn._etag_handler(path=target_uri, headers=self.headers,
+                                data=data, etag=self.headers['If-Match'],
+                                blocking=False, timeout=60)
+        mock__op.assert_called_with(
+            self.conn,
+            "PATCH",
+            '/redfish/v1/Systems/1',
+            data={'Boot': {'BootSourceOverrideTarget': 'Cd',
+                           'BootSourceOverrideEnabled': 'Once'}},
+            headers=self.headers_no_etag,
+            blocking=False,
+            timeout=60)
+
     @mock.patch.object(connector.Connector, '_op', autospec=True)
     def test_etag_handler_fail_other_exception(self, mock__op):
         self.headers['If-Match'] = 'W/"3d7b8a7360bf2941d"'
@@ -912,3 +960,25 @@ class ConnectorOpTestCase(base.TestCase):
                           'PATCH', target_uri, data,
                           self.headers,
                           blocking=False, timeout=60)
+
+    @mock.patch.object(time, 'sleep', autospec=True)
+    def test_retry_on_connection_error(self, mock_sleep):
+        self.request.side_effect = [
+            requests.exceptions.ConnectionError("temporary issue"),
+            mock.Mock(status_code=200, headers={}, json=lambda: {})
+        ]
+
+        target_uri = '/redfish/v1/Systems/1'
+        result = self.conn.get(target_uri)
+        self.assertEqual(result.status_code, 200)
+        self.assertEqual(self.request.call_count, 2)
+
+    @mock.patch.object(time, 'sleep', autospec=True)
+    def test_raises_after_max_retries(self, mock_sleep):
+        target_uri = '/redfish/v1/Systems/1'
+        self.request.side_effect = requests.exceptions.ConnectTimeout(
+            "timeout")
+        self.assertRaises(exceptions.ConnectionError, self.conn.get,
+                          target_uri)
+        self.assertEqual(self.request.call_count,
+                         self.conn._server_side_retries)
